@@ -25,10 +25,20 @@ struct _JammoCursorPrivate {
 	gulong sequencer_started_handler_id;
 	gulong sequencer_stopped_handler_id;
 	guint timeout_source_id;
+	
+	gfloat seek_start_stage_x;
+	gfloat seek_start_texture_x;
+	guint motion_event_handler_id;
+	guint button_release_event_handler_id;
+	
+	guint seeking : 1;
 };
 
 static void on_sequencer_started(JammoSequencer* sequencer, gpointer user_data);
 static void on_sequencer_stopped(JammoSequencer* sequencer, gpointer user_data);
+static gboolean on_button_press_event(ClutterActor* actor, ClutterEvent* event, gpointer user_data);
+static gboolean on_motion_event(ClutterActor* actor, ClutterEvent* event, gpointer user_data);
+static gboolean on_button_release_event(ClutterActor* actor, ClutterEvent* event, gpointer user_data);
 
 ClutterActor* jammo_cursor_new(JammoSequencer* sequencer, ClutterActor* texture) {
 
@@ -37,12 +47,12 @@ ClutterActor* jammo_cursor_new(JammoSequencer* sequencer, ClutterActor* texture)
 
 static void jammo_cursor_dispose(GObject* object) {
 	JammoCursor* cursor;
-	
+
 	cursor = JAMMO_CURSOR(object);
-	
+
 	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(cursor->priv->sequencer);
 	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(cursor->priv->texture);
-	
+
 	if (cursor->priv->sequencer_started_handler_id) {
 		g_signal_handler_disconnect(cursor->priv->sequencer, cursor->priv->sequencer_started_handler_id);
 		cursor->priv->sequencer_started_handler_id = 0;
@@ -51,7 +61,12 @@ static void jammo_cursor_dispose(GObject* object) {
 		g_signal_handler_disconnect(cursor->priv->sequencer, cursor->priv->sequencer_stopped_handler_id);
 		cursor->priv->sequencer_stopped_handler_id = 0;
 	}
-	
+	if (cursor->priv->motion_event_handler_id) {
+		g_signal_handler_disconnect(clutter_actor_get_stage(CLUTTER_ACTOR(cursor)), cursor->priv->motion_event_handler_id);
+		g_signal_handler_disconnect(clutter_actor_get_stage(CLUTTER_ACTOR(cursor)), cursor->priv->button_release_event_handler_id);
+		cursor->priv->motion_event_handler_id = cursor->priv->button_release_event_handler_id = 0;
+	}
+
 	G_OBJECT_CLASS(jammo_cursor_parent_class)->dispose(object);
 }
 
@@ -70,10 +85,13 @@ static void jammo_cursor_set_property(GObject* object, guint prop_id, const GVal
 		case PROP_TEXTURE:
 			if (cursor->priv->texture) {
 				clutter_container_remove_actor(CLUTTER_CONTAINER(cursor), cursor->priv->texture);
+				g_signal_handlers_disconnect_by_func(cursor->priv->texture, G_CALLBACK(on_button_press_event), cursor);
 				g_object_unref(cursor->priv->texture);
 			}
 			cursor->priv->texture = CLUTTER_ACTOR(g_value_get_object(value));
+			clutter_actor_set_reactive(CLUTTER_ACTOR(cursor->priv->texture), TRUE);
 			clutter_container_add_actor(CLUTTER_CONTAINER(cursor), cursor->priv->texture);
+			g_signal_connect(cursor->priv->texture, "button-press-event", G_CALLBACK(on_button_press_event), cursor);
 			g_object_ref(cursor->priv->texture);
 			clutter_actor_hide(cursor->priv->texture);
 			break;
@@ -135,7 +153,7 @@ static void jammo_cursor_class_init(JammoCursorClass* cursor_class) {
 
 static void jammo_cursor_init(JammoCursor* cursor) {
 	cursor->priv = G_TYPE_INSTANCE_GET_PRIVATE(cursor, JAMMO_TYPE_CURSOR, JammoCursorPrivate);
-	
+
 	g_object_set(G_OBJECT(cursor), "pickable", FALSE, NULL);
 }
 
@@ -147,7 +165,7 @@ static gboolean on_timeout(gpointer user_data) {
 	
 	cursor = JAMMO_CURSOR(user_data);
 	
-	if (cursor->priv->texture) {
+	if (cursor->priv->texture && !cursor->priv->seeking) {
 		position = jammo_sequencer_get_position(cursor->priv->sequencer) / 1000000;
 		duration = jammo_sequencer_get_duration(cursor->priv->sequencer) / 1000000;
 		fraction =  (gfloat)position / duration;
@@ -188,3 +206,63 @@ static void on_sequencer_stopped(JammoSequencer* sequencer, gpointer user_data)
 		cursor->priv->timeout_source_id = 0;
 	}
 }
+
+static gboolean on_button_press_event(ClutterActor* actor, ClutterEvent* event, gpointer user_data) {
+	JammoCursor* cursor;
+	TangleEvent* tangle_event;
+	
+	cursor = JAMMO_CURSOR(user_data);
+	
+	cursor->priv->seeking = TRUE;
+	clutter_event_get_coords(event, &cursor->priv->seek_start_stage_x, NULL);
+	cursor->priv->seek_start_texture_x = clutter_actor_get_x(cursor->priv->texture);
+
+	cursor->priv->motion_event_handler_id = g_signal_connect(clutter_actor_get_stage(actor), "motion-event", G_CALLBACK(on_motion_event), cursor);
+	cursor->priv->button_release_event_handler_id = g_signal_connect(clutter_actor_get_stage(actor), "button-release-event", G_CALLBACK(on_button_release_event), cursor);
+	
+	tangle_event = tangle_event_new(TANGLE_EVENT_INTERACTION_BEGAN, TANGLE_ACTOR(cursor));
+	tangle_event->interaction.x_axis = TRUE;
+	tangle_event->interaction.y_axis = TRUE;
+	tangle_actor_send_event(TANGLE_ACTOR(cursor), tangle_event);
+	tangle_event_free(tangle_event);
+
+	return FALSE;
+}
+
+static gboolean on_motion_event(ClutterActor* actor, ClutterEvent* event, gpointer user_data) {
+	JammoCursor* cursor;
+	gfloat stage_x, texture_x;
+	
+	cursor = JAMMO_CURSOR(user_data);
+	
+	clutter_event_get_coords(event, &stage_x, NULL);
+	texture_x = cursor->priv->seek_start_texture_x + stage_x - cursor->priv->seek_start_stage_x;
+	clutter_actor_set_x(cursor->priv->texture, texture_x);
+	
+	return FALSE;
+}
+
+static gboolean on_button_release_event(ClutterActor* actor, ClutterEvent* event, gpointer user_data) {
+	JammoCursor* cursor;
+	gfloat stage_x, texture_x;
+	gfloat fraction; 
+	
+	cursor = JAMMO_CURSOR(user_data);
+
+	cursor->priv->seeking = FALSE;
+
+	clutter_event_get_coords(event, &stage_x, NULL);
+	texture_x = cursor->priv->seek_start_texture_x + stage_x - cursor->priv->seek_start_stage_x;
+	clutter_actor_set_x(cursor->priv->texture, texture_x);
+	
+	fraction = texture_x / (clutter_actor_get_width(CLUTTER_ACTOR(cursor)) - clutter_actor_get_width(cursor->priv->texture) / 2);
+	jammo_sequencer_set_position(cursor->priv->sequencer, jammo_sequencer_get_duration(cursor->priv->sequencer) * fraction);
+	
+	if (cursor->priv->motion_event_handler_id) {
+		g_signal_handler_disconnect(actor, cursor->priv->motion_event_handler_id);
+		g_signal_handler_disconnect(actor, cursor->priv->button_release_event_handler_id);
+		cursor->priv->motion_event_handler_id = cursor->priv->button_release_event_handler_id = 0;
+	}
+	
+	return FALSE;
+}
